-------------------------------------------------------------
--	Filename:  COM2001TEST.VHD
-- Author: 
--		Alain Zarembowitch / MSS, 
--	Version: 0
--	Date last modified: 9-29-10
-- Inheritance: 	none
--
-- description:  Hardware verification test for the COM-1600 (or COM-1500) + COM-2001 assembly.
-- The COM-2001 plug-in module includes two 10-bit 125 MSamples/s DACs.
-- Test pattern consists of two ramps in opposite directions. Ramp frequency: CLK/1024
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
library UNISIM;
use UNISIM.VComponents.all;

entity COM2001TEST is
	generic (
		WAVEFORM: in integer := 1		-- 1 = two ramps in opposite directions
												-- 2 = sine/cosine
	);
    port ( 
			--// Clocks Resets
			CLK: in std_logic;
				-- reference clock. Maximum 125 MHz
			
			--// controls
			CARRIER_FREQ: in std_logic_vector(31 downto 0);
				-- carrier frequency for sine/cosine

			--// COM-2001 125 MS/s 10-bit output
			DAC1_DOUT: out std_logic_vector(9 downto 0);
			DAC2_DOUT: out std_logic_vector(9 downto 0);
				-- format: 10-bit unsigned
			DAC_CLKOUT1: out std_logic;
			DAC_CLKOUT2: out std_logic
				-- two identical clock outputs for pins A1 and A13
			
		
			);
end entity;

architecture Behavioral of COM2001TEST is
--------------------------------------------------------
--      COMPONENTS
--------------------------------------------------------
	COMPONENT NCO32
	PORT(
		CLK : IN std_logic;
		SYNC_RESET : IN std_logic;
		NOMINAL_FREQUENCY : IN std_logic_vector(31 downto 0);
		PHASE_OFFSET : IN std_logic_vector(31 downto 0);
		PHASE_OFFSET_SAMPLE_CLK : IN std_logic;          
		PHASE : OUT std_logic_vector(31 downto 0)
		);
	END COMPONENT;

	COMPONENT SIGNED_SIN_COS_TBL2
	PORT(
		CLK : IN std_logic;
		PHASE : IN std_logic_vector(11 downto 0);          
		SIN_OUT : OUT std_logic_vector(17 downto 0);
		COS_OUT : OUT std_logic_vector(17 downto 0)
		);
	END COMPONENT;
--------------------------------------------------------
--     SIGNALS
--------------------------------------------------------
-- Suffix _D indicates a one clock delay
signal DAC1_DOUT_LOCAL: std_logic_vector(9 downto 0);
signal DAC2_DOUT_LOCAL: std_logic_vector(9 downto 0);
signal PHASE: std_logic_vector(31 downto 0);
signal SIN_OUT: std_logic_vector(17 downto 0);
signal COS_OUT: std_logic_vector(17 downto 0);
--------------------------------------------------------
--      IMPLEMENTATION
--------------------------------------------------------
begin


--// DAC data stimulus (for test purposes)
-- generate two ramps in opposite direction
RAMP_GEN: if(WAVEFORM = 1) generate
	DACGEN: process(CLK)
	begin
		if rising_edge(CLK) then
			DAC1_DOUT_LOCAL <= DAC1_DOUT_LOCAL + 1;
			DAC2_DOUT_LOCAL <= DAC2_DOUT_LOCAL - 1;
		end if;
	end process;
	DAC1_DOUT <= DAC1_DOUT_LOCAL;
	DAC2_DOUT <= DAC2_DOUT_LOCAL;
end generate;

-- generate 90deg apart sin/cos
SINCOS_GEN: if (WAVEFORM = 2) generate
	Inst_NCO32: NCO32 PORT MAP(
		CLK => CLK,
		SYNC_RESET => '0',
		NOMINAL_FREQUENCY => CARRIER_FREQ,
		PHASE_OFFSET => x"00000000",
		PHASE_OFFSET_SAMPLE_CLK => '0',
		PHASE => PHASE
	);

	Inst_SIGNED_SIN_COS_TBL2: SIGNED_SIN_COS_TBL2 PORT MAP(
		CLK => CLK,
		PHASE => PHASE(31 downto 20),
		SIN_OUT => SIN_OUT,
		COS_OUT => COS_OUT
	);
	
	DAC1_DOUT <= (not SIN_OUT(17)) & SIN_OUT(16 downto 8);	-- format is unsigned at COM-2001
	DAC2_DOUT <= (not COS_OUT(17)) & COS_OUT(16 downto 8);
end generate;

-- clock forwarding
-- Invert clock so that the rising edge is at the center of the data sample where it is most stable.
DAC_CLKOUT_GEN_001 : ODDR2
generic map(
	DDR_ALIGNMENT => "NONE", -- Sets output alignment to "NONE", "C0", "C1" 
	INIT => '0', -- Sets initial state of the Q output to '0' or '1'
	SRTYPE => "SYNC") -- Specifies "SYNC" or "ASYNC" set/reset
port map (
	Q => DAC_CLKOUT1, -- 1-bit output data
	C0 => CLK, -- 1-bit clock input
	C1 => not CLK, -- 1-bit clock input
	CE => '1',  -- 1-bit clock enable input
	D0 => '0',   -- 1-bit data input (associated with C0)
	D1 => '1',   -- 1-bit data input (associated with C1)
	R => '0',    -- 1-bit reset input
	S => '0'     -- 1-bit set input
);

DAC_CLKOUT_GEN_002 : ODDR2
generic map(
	DDR_ALIGNMENT => "NONE", -- Sets output alignment to "NONE", "C0", "C1" 
	INIT => '0', -- Sets initial state of the Q output to '0' or '1'
	SRTYPE => "SYNC") -- Specifies "SYNC" or "ASYNC" set/reset
port map (
	Q => DAC_CLKOUT2, -- 1-bit output data
	C0 => CLK, -- 1-bit clock input
	C1 => not CLK, -- 1-bit clock input
	CE => '1',  -- 1-bit clock enable input
	D0 => '0',   -- 1-bit data input (associated with C0)
	D1 => '1',   -- 1-bit data input (associated with C1)
	R => '0',    -- 1-bit reset input
	S => '0'     -- 1-bit set input
);


end Behavioral;

