-------------------------------------------------------------
-- MSS copyright 2010
--	Filename:  tbDDR2_SDRAM_CONTROLLER.VHD
-- Author: Alain Zarembowitch / MSS
--	Version: 1
--	Date last modified: 9-21-10
-- Inheritance: 	N/A
--
-- description: Testbench for the DDR2_SDRAM_CONTROLLER_TEST component. 
-- The DDR2_MODULE component is Micron SDRAM DDR2 (Double Data Rate 2) module model.
-- The DDR2_SDRAM_CONTROLLER_TEST checks the SDRAM by writing to a range of addresses, 
-- then reads back the data from the same address range to compare with the original data.
---------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_unsigned.all;
USE ieee.numeric_std.ALL;
Library UNISIM;
use UNISIM.vcomponents.all;
 
ENTITY tbDDR2_SDRAM_CONTROLLER_TEST IS
END entity;
 
ARCHITECTURE behavior OF tbDDR2_SDRAM_CONTROLLER_TEST IS 
 
-----------------------------------------------------------------
-- COMPONENTS
-----------------------------------------------------------------
	COMPONENT ddr2_module
	PORT(
		ck : IN std_logic_vector(1 downto 0);
		ck_n : IN std_logic_vector(1 downto 0);
		cke : IN std_logic_vector(1 downto 0);
		s_n : IN std_logic_vector(1 downto 0);
		ras_n : IN std_logic;
		cas_n : IN std_logic;
		we_n : IN std_logic;
		ba : IN std_logic_vector(2 downto 0);
		addr : IN std_logic_vector(15 downto 0);
		odt : IN std_logic_vector(1 downto 0);
		scl : IN std_logic;
		sa : IN std_logic_vector(1 downto 0);       
		dqs : INOUT std_logic_vector(17 downto 0);
		dqs_n : INOUT std_logic_vector(17 downto 0);
		dq : INOUT std_logic_vector(63 downto 0);
		sda : INOUT std_logic
		);
	END COMPONENT;

	COMPONENT ddr2_sdram_controller_test
	Generic (

	NSDRAM_BYTES: in integer;
		-- number of data bytes on the SDRAM data bus 
		-- The same data width for both user data interface and DDR2 interface
		-- 1 = 8-bit word (IC)
		-- 2 = 16-bit word (IC, e.g. COM-1600)
		-- 4 = 32-bit word (ICs or partial DIMM/SODIMM)
		-- 8 = 64-bit word (DIMM/SODIMM, e.g. COM-1500)
	NCLKS: in integer := 1;
		-- number of differential clock pairs: 1 for IC, 2 for SODIMM
	ADDRESS_MAP: in std_logic_vector(4 downto 0);
		-- the logical to physical address mapping can vary depending on the hardware.
		-- See DDR2 SDRAM hardware specifications.
		-- the most common mappings are described below:
		-- 00000: 1Gb x8 IC, column A[9:0], 8 banks BA[2:0], row A[13:0], 1 rank CS# 
		-- 00001: 1Gb x16 IC, column A[9:0], 8 banks BA[2:0], row A[12:0], 1 rank CS#
		-- 00010: 2Gb x8 IC, column A[9:0], 8 banks BA[2:0], row A[14:0], 1 rank CS#
		-- 00011: 2Gb x16 IC, column A[9:0], 8 banks BA[2:0], row A[13:0], 1 rank CS#
		-- 0xxxx: other ICs, reserved for future use
		-- 10000: 1GB DIMM/SODIMM, column A[9:0], 8 banks BA[2:0], row A[12:0], 2 ranks S0#/S1#
		-- 10001: 2GB DIMM/SODIMM, column A[9:0], 8 banks BA[2:0], row A[13:0], 2 ranks S0#/S1#
		-- 10010: 4GB DIMM/SODIMM, column A[9:0], 8 banks BA[2:0], row A[14:0], 2 ranks S0#/S1#
		-- 1xxxx: other DIMM/SODIMMs, reserved for future use
	CLOCK_CYCLES: in integer;
		-- (CAS latency) -  (RAS-CAS-Delay time) -  (RAS precharge time)
		-- CL - tRCD - tRP
		-- See DDR2 SDRAM hardware specifications.
		-- 3 =  3-3-3  DDR2-400
		-- 4 =  4-4-4  DDR2-533
		-- 5 =  5-5-5  DDR2-667
		-- 6 =  6-6-6  DDR2-800
	DDR2_CLK_FREQ: in integer;
		-- actual DDR2 clock frequency in 100KHz units (used to calculate timers within)
		-- For example: 2000 means a 200 MHz clock and a maximum throughput of 400 Mtransactions/s.
		-- The specified value can be significantly slower than the SDRAM DDR2 speed grade 
		-- (since the FPGA may be the bottleneck)
	
	RTDELAY_COARSE: in integer
		-- goal: calibrate out signal round-trip delay. 
		-- Coarse settings: integer multiple of the CLK period rounded down. 
	);
	PORT(
		ASYNC_RESET : IN std_logic;
		CLK_DDR2_REF : IN std_logic;
		CLK_DDR2G_OUT: out std_logic;
		ADDR_MIN : IN std_logic_vector(31 downto 0);
		ADDR_MAX : IN std_logic_vector(31 downto 0);    
		DONE: out std_logic;
		SUCCESS: out std_logic;
		ERROR: out std_logic;
		SDRAM_DQ : INOUT std_logic_vector((8*NSDRAM_BYTES - 1) downto 0);
		SDRAM_DQS_P : INOUT std_logic_vector((NSDRAM_BYTES - 1) downto 0);
		SDRAM_DQS_N : INOUT std_logic_vector((NSDRAM_BYTES - 1) downto 0);    
		SDRAM_ODT : OUT std_logic_vector(1 downto 0);
		SDRAM_CK_P: out std_logic_vector((NCLKS - 1) downto 0);
		SDRAM_CK_N: out std_logic_vector((NCLKS - 1) downto 0);
		SDRAM_CKE: out std_logic_vector((NCLKS - 1) downto 0);
		SDRAM_CASN : OUT std_logic;
		SDRAM_RASN : OUT std_logic;
		SDRAM_WEN : OUT std_logic;
		SDRAM_SN : OUT std_logic_vector(1 downto 0);
		SDRAM_DM : OUT std_logic_vector((NSDRAM_BYTES - 1) downto 0);
		SDRAM_BA : OUT std_logic_vector(2 downto 0);
		SDRAM_A : OUT std_logic_vector(15 downto 0);
		SDRAM_DQ_DIR: out std_logic;
		SDRAM_DQS_DIR: out std_logic;
		TEST_POINTS : OUT std_logic_vector(10 downto 1)
		);
	END COMPONENT;
    

--------------------------------------------------------
--     SIGNALS
--------------------------------------------------------
--constant NSDRAM_BYTES: integer := 8;	-- 64-bit wide data bus 
constant NSDRAM_BYTES: integer := 2;	-- 16-bit wide data bus
constant NCLKS: integer := 1;				-- 1 CLK
--constant CLK_period : time := 5 ns;  -- Clock period  200 MHz  
constant CLK_period : time := 8 ns;  -- Clock period  125 MHz  
constant DELAY_SDRAM2FPGA: time := 7.0 ns;	-- FPGA input delay


   signal ASYNC_RESET : std_logic := '0';
	signal SYNC_COUNTER: std_logic_vector(3 downto 0) := "0000";
	signal CLK_DDR2_REF: std_logic := '0';
	signal CLK: std_logic := '0';

	--BiDirs
   signal SDRAM_DQ : std_logic_vector(63 downto 0);
   signal SDRAM_DQS_P : std_logic_vector(17 downto 0);
   signal SDRAM_DQS_N : std_logic_vector(17 downto 0);
	-- same as above but delayed
   signal SDRAM_DQ_D : std_logic_vector(63 downto 0);
   signal SDRAM_DQS_P_D : std_logic_vector(17 downto 0);
   signal SDRAM_DQS_N_D : std_logic_vector(17 downto 0);

 	--Outputs
   signal SDRAM_ODT : std_logic_vector(1 downto 0) := (others => '0');
   signal SDRAM_CK_P : std_logic_vector(1 downto 0) := (others => '0');
   signal SDRAM_CK_N : std_logic_vector(1 downto 0) := (others => '0');
   signal SDRAM_CKE : std_logic_vector(1 downto 0) := (others => '0');
   signal SDRAM_CASN : std_logic := '0';
   signal SDRAM_RASN : std_logic := '0';
   signal SDRAM_WEN : std_logic := '0';
   signal SDRAM_SN : std_logic_vector(1 downto 0) := (others => '0');
   signal SDRAM_DM : std_logic_vector(7 downto 0) := (others => '0');
   signal SDRAM_BA : std_logic_vector(2 downto 0) := (others => '0');
   signal SDRAM_A : std_logic_vector(15 downto 0) := (others => '0');
   signal SDRAM_TP : std_logic_vector(10 downto 1) := (others => '0');
	
	-- DDR2 module
	signal SDA: std_logic;
	
	-- delayed IO signals
  signal SDRAM_DQS_P_FPGA : std_logic_vector(17 downto 0) := (others => '0');
  signal SDRAM_DQS_N_FPGA : std_logic_vector(17 downto 0) := (others => '0');
  signal SDRAM_DQ_FPGA : std_logic_vector(63 downto 0) := (others => '0');
  signal SDRAM_DQ_DIR: std_logic := '0';
  signal SDRAM_DQS_DIR: std_logic := '0';
	

 
-----------------------------------------------------------------
-- IMPLEMENTATION
-----------------------------------------------------------------
BEGIN

-- generate reference clock 
CLK_DDR2_REF_process :process
begin
	CLK_DDR2_REF <= '0';
	wait for CLK_period/2;
	CLK_DDR2_REF <= '1';
	wait for CLK_period/2;
end process;

ASYNC_RESET <= '0';	-- reset not mandatory.


 
			 
	  
Inst_ddr2_sdram_controller_test: ddr2_sdram_controller_test 
generic map(
--	NSDRAM_BYTES => NSDRAM_BYTES,	-- 8 = sodimm 8-byte wide data bus
--	NCLKS => 2,					-- 2 CLKs (SODIMM)
--	ADDRESS_MAP => "10010",	-- 4GB SODIMM module
--	CLOCK_CYCLES => 3,		-- CL
--	DDR2_CLK_FREQ => 2000,	-- 200 MHz
--	RTDELAY_COARSE => 2	-- ideal no round-trip delay fpga-sodimm-fpga
	NSDRAM_BYTES => NSDRAM_BYTES,	-- 2 = sodimm 8-byte wide data bus
	NCLKS => 1,					-- 1 CLK
	ADDRESS_MAP => "00001",	-- 1Gb 16-bit wide IC
	CLOCK_CYCLES => 3,		-- CL
	DDR2_CLK_FREQ => 1250,	-- 125 MHz
	RTDELAY_COARSE => 2	-- ideal no round-trip delay fpga-sodimm-fpga
)
PORT MAP(
	ASYNC_RESET => ASYNC_RESET,
	CLK_DDR2_REF => CLK_DDR2_REF,
	CLK_DDR2G_OUT => CLK,
	ADDR_MIN => x"00000400",
	ADDR_MAX => x"00000410",
	DONE => open,
	SUCCESS => open,
	ERROR => open,
	SDRAM_DQ => SDRAM_DQ_FPGA((8*NSDRAM_BYTES - 1) downto 0),
	SDRAM_DQS_P => SDRAM_DQS_P_FPGA((NSDRAM_BYTES - 1) downto 0),
	SDRAM_DQS_N => SDRAM_DQS_N_FPGA((NSDRAM_BYTES - 1) downto 0),
	SDRAM_ODT => SDRAM_ODT,
	SDRAM_CK_P => SDRAM_CK_P((NCLKS -1) downto 0),
	SDRAM_CK_N => SDRAM_CK_N((NCLKS -1) downto 0),
	SDRAM_CKE => SDRAM_CKE((NCLKS -1) downto 0),
	SDRAM_CASN => SDRAM_CASN,
	SDRAM_RASN => SDRAM_RASN ,
	SDRAM_WEN => SDRAM_WEN,
	SDRAM_SN => SDRAM_SN,
	SDRAM_DM => SDRAM_DM((NSDRAM_BYTES - 1) downto 0),
	SDRAM_BA => SDRAM_BA,
	SDRAM_A => SDRAM_A,
	SDRAM_DQ_DIR => SDRAM_DQ_DIR,
	SDRAM_DQS_DIR => SDRAM_DQS_DIR,
	TEST_POINTS => SDRAM_TP 
);

-- simulate delay at FPGA input
SDRAM_DQS_P_D((NSDRAM_BYTES - 1) downto 0) <= transport SDRAM_DQS_P((NSDRAM_BYTES - 1) downto 0) after DELAY_SDRAM2FPGA;
SDRAM_DQS_N_D((NSDRAM_BYTES - 1) downto 0) <= transport SDRAM_DQS_N((NSDRAM_BYTES - 1) downto 0) after DELAY_SDRAM2FPGA;
SDRAM_DQ_D((8*NSDRAM_BYTES - 1) downto 0) <= transport SDRAM_DQ((8*NSDRAM_BYTES - 1) downto 0) after DELAY_SDRAM2FPGA;

process(SDRAM_DQ_DIR, SDRAM_DQ_FPGA, SDRAM_DQ_D)
begin
	if(SDRAM_DQ_DIR = '0') then
		-- FPGA to SDRAM direction: no delay (all signals and clocks going in the same direction)
		SDRAM_DQ((8*NSDRAM_BYTES - 1) downto 0) <= SDRAM_DQ_FPGA((8*NSDRAM_BYTES - 1) downto 0);
		SDRAM_DQ_FPGA((8*NSDRAM_BYTES - 1) downto 0) <= (others => 'Z');
	else
		-- SDRAM to FPGA direction. delay.
		SDRAM_DQ((8*NSDRAM_BYTES - 1) downto 0) <= (others => 'Z');
		SDRAM_DQ_FPGA((8*NSDRAM_BYTES - 1) downto 0) <= SDRAM_DQ_D((8*NSDRAM_BYTES - 1) downto 0);
	end if;
end process;

process(SDRAM_DQS_DIR, SDRAM_DQS_P_FPGA, SDRAM_DQS_N_FPGA, SDRAM_DQS_P_D, SDRAM_DQS_N_D)
begin
	if(SDRAM_DQS_DIR = '0') then
		-- FPGA to SDRAM direction: no delay (all signals and clocks going in the same direction)
		SDRAM_DQS_P((NSDRAM_BYTES - 1) downto 0) <=  SDRAM_DQS_P_FPGA((NSDRAM_BYTES - 1) downto 0);
		SDRAM_DQS_N((NSDRAM_BYTES - 1) downto 0) <=  SDRAM_DQS_N_FPGA((NSDRAM_BYTES - 1) downto 0);
		SDRAM_DQS_P_FPGA((NSDRAM_BYTES - 1) downto 0) <=  (others => 'Z');
		SDRAM_DQS_N_FPGA((NSDRAM_BYTES - 1) downto 0) <=  (others => 'Z');
	else
		-- SDRAM to FPGA direction. delay.
		SDRAM_DQS_P((NSDRAM_BYTES - 1) downto 0) <=  (others => 'Z');
		SDRAM_DQS_N((NSDRAM_BYTES - 1) downto 0) <=  (others => 'Z');
		SDRAM_DQS_P_FPGA((NSDRAM_BYTES - 1) downto 0) <=  SDRAM_DQS_P_D((NSDRAM_BYTES - 1) downto 0);
		SDRAM_DQS_N_FPGA((NSDRAM_BYTES - 1) downto 0) <=  SDRAM_DQS_N_D((NSDRAM_BYTES - 1) downto 0);
	end if;
end process;




--// DDR2 MODULE SIMULATION (MICRON)
SDRAM_DQS_P(17 downto NSDRAM_BYTES) <= (others => '0');
SDRAM_DQS_N(17 downto NSDRAM_BYTES) <= (others => '0');

Inst_ddr2_module: ddr2_module PORT MAP(
	ck => SDRAM_CK_P,
	ck_n => SDRAM_CK_N,
	cke => SDRAM_CKE,
	s_n => SDRAM_SN,
	ras_n => SDRAM_RASN,
	cas_n => SDRAM_CASN,
	we_n => SDRAM_WEN,
	ba => SDRAM_BA,
	addr => SDRAM_A,
	odt => SDRAM_ODT,
	dqs => SDRAM_DQS_P,
	dqs_n => SDRAM_DQS_N,
	dq => SDRAM_DQ,
	scl => '0',
	sa => "00",
	sda => SDA
);


END behavior;
