-------------------------------------------------------------
-- MSS copyright 2010-2012
--	Filename:  tbDDR2_WRAPPERx64b.VHD
-- Author: Alain Zarembowitch / MSS
--	Version: 2
--	Date last modified: 11/10/12
-- Inheritance: 	N/A
--
-- description: Testbench for the DDR2_WRAPPER component  (64-bit wide SDRAM version) 
-- Writes then reads back a short sequence of data in the specified address range (start/ceiling).
-- The DDR2_MODULE component is Micron SDRAM DDR2 (Double Data Rate 2) module model.
-- Select the memory type (16-bit wide, 64-bit DIMM, etc) by commenting and uncommenting the
-- constants within.
-- The simulation must run for at least 235us to give enough time for the SDRAM intialization.
--
-- Rev 2 11/10/12 AZ
-- Added platform generic parameter
---------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_unsigned.all;
USE ieee.numeric_std.ALL;
 
ENTITY tbDDR2_WRAPPERx64b IS
END tbDDR2_WRAPPERx64b;
 
ARCHITECTURE behavior OF tbDDR2_WRAPPERx64b IS 
 
--------------------------------------------------------
--      COMPONENTS
--------------------------------------------------------
	COMPONENT ddr2_module
	PORT(
		ck : IN std_logic_vector(1 downto 0);
		ck_n : IN std_logic_vector(1 downto 0);
		cke : IN std_logic_vector(1 downto 0);
		s_n : IN std_logic_vector(1 downto 0);
		ras_n : IN std_logic;
		cas_n : IN std_logic;
		we_n : IN std_logic;
		ba : IN std_logic_vector(2 downto 0);
		addr : IN std_logic_vector(15 downto 0);
		odt : IN std_logic_vector(1 downto 0);
		scl : IN std_logic;
		sa : IN std_logic_vector(1 downto 0);       
		dqs : INOUT std_logic_vector(17 downto 0);
		dqs_n : INOUT std_logic_vector(17 downto 0);
		dq : INOUT std_logic_vector(63 downto 0);
		sda : INOUT std_logic
		);
	END COMPONENT;

    COMPONENT DDR2_WRAPPER
	Generic (
	PLATFORM: integer := 5;
		-- 5 = Xilinx Virtex-5
		-- 6 = Xilinx Spartan-6
	NSDRAM_BYTES: in integer;
		-- number of data bytes on the SDRAM data bus 
		-- The same data width for both user data interface and DDR2 interface
		-- 1 = 8-bit word (IC)
		-- 2 = 16-bit word (IC, e.g. COM-1600)
		-- 4 = 32-bit word (ICs or partial DIMM/SODIMM)
		-- 8 = 64-bit word (DIMM/SODIMM, e.g. COM-1500)
	NCLKS: in integer := 1;
		-- number of differential clock pairs: 1 for IC, 2 for SODIMM
	ADDRESS_MAP: in std_logic_vector(4 downto 0);
		-- the logical to physical address mapping can vary depending on the hardware.
		-- See DDR2 SDRAM hardware specifications.
		-- the most common mappings are described below:
		-- 00000: 1Gb x8 IC, column A[9:0], 8 banks BA[2:0], row A[13:0], 1 rank CS# 
		-- 00001: 1Gb x16 IC, column A[9:0], 8 banks BA[2:0], row A[12:0], 1 rank CS#
		-- 00010: 2Gb x8 IC, column A[9:0], 8 banks BA[2:0], row A[14:0], 1 rank CS#
		-- 00011: 2Gb x16 IC, column A[9:0], 8 banks BA[2:0], row A[13:0], 1 rank CS#
		-- 0xxxx: other ICs, reserved for future use
		-- 10000: 1GB DIMM/SODIMM, column A[9:0], 8 banks BA[2:0], row A[12:0], 2 ranks S0#/S1#
		-- 10001: 2GB DIMM/SODIMM, column A[9:0], 8 banks BA[2:0], row A[13:0], 2 ranks S0#/S1#
		-- 10010: 4GB DIMM/SODIMM, column A[9:0], 8 banks BA[2:0], row A[14:0], 2 ranks S0#/S1#
		-- 1xxxx: other DIMM/SODIMMs, reserved for future use
	CLOCK_CYCLES: in integer;
		-- (CAS latency) -  (RAS-CAS-Delay time) -  (RAS precharge time)
		-- CL - tRCD - tRP
		-- See DDR2 SDRAM hardware specifications.
		-- 3 =  3-3-3  DDR2-400
		-- 4 =  4-4-4  DDR2-533
		-- 5 =  5-5-5  DDR2-667
		-- 6 =  6-6-6  DDR2-800
	DDR2_CLK_FREQ: in integer;
		-- actual DDR2 clock frequency in 100KHz units (used to calculate timers within)
		-- For example: 2000 means a 200 MHz clock and a maximum throughput of 400 Mtransactions/s.
		-- The specified value can be significantly slower than the SDRAM DDR2 speed grade 
		-- (since the FPGA may be the bottleneck)
	
	RTDELAY_COARSE: in integer
		-- goal: calibrate out signal round-trip delay. 
		-- Coarse settings: integer multiple of the CLK period rounded down. 
	);
    PORT(
		CLK: in std_logic;
		CLK_DDR2_REF: in std_logic; 
		ASYNC_RESET: in std_logic;
		DATA1_IN: in std_logic_vector((16*NSDRAM_BYTES - 1) downto 0);
		DATA1_IN_EN: in std_logic_vector(1 downto 0);
		DATA1_IN_CTS: out std_logic;
		DATA1_IN_SETADDR: in std_logic;
		DATA1_IN_STARTADDR: in std_logic_vector(31 downto 0);
		DATA1_IN_CEILINGADDR: in std_logic_vector(31 downto 0);
		DATA1_IN_MODE: in std_logic_vector(1 downto 0);
		DATA1_OUT: out std_logic_vector((16*NSDRAM_BYTES - 1) downto 0);
		DATA1_OUT_VALID: out std_logic;
		DATA1_OUT_CTS: in std_logic;
		DATA1_OUT_EMPTY: out std_logic;
		DATA1_OUT_SETADDR: in std_logic;
		DATA1_OUT_STARTADDR: in std_logic_vector(31 downto 0);
		DATA1_OUT_CEILINGADDR: in std_logic_vector(31 downto 0);
		DATA1_OUT_MODE: in std_logic_vector(1 downto 0);
		DATA1_IN_ADDR: out std_logic_vector(31 downto 0);
		DATA1_OUT_ADDR: out std_logic_vector(31 downto 0);
		SDRAM_DQ: inout std_logic_vector((8*NSDRAM_BYTES - 1) downto 0);
		SDRAM_DQS_P: inout std_logic_vector((NSDRAM_BYTES - 1) downto 0);
		SDRAM_DQS_N: inout std_logic_vector((NSDRAM_BYTES - 1) downto 0);
		SDRAM_ODT: out std_logic_vector(1 downto 0);
		SDRAM_CK_P: out std_logic_vector((NCLKS - 1) downto 0);
		SDRAM_CK_N: out std_logic_vector((NCLKS - 1) downto 0);
		SDRAM_CKE: out std_logic_vector((NCLKS - 1) downto 0);
		SDRAM_CASN: out std_logic;
		SDRAM_RASN: out std_logic;
		SDRAM_WEN: out std_logic;
		SDRAM_SN: out std_logic_vector(1 downto 0);
		SDRAM_DM: out std_logic_vector((NSDRAM_BYTES - 1) downto 0);
		SDRAM_BA: out std_logic_vector(2 downto 0);
		SDRAM_A: out std_logic_vector(15 downto 0); 
		SDRAM_DQ_DIR: out std_logic;
		SDRAM_DQS_DIR: out std_logic;
		SDRAM_TP: out std_logic_vector(10 downto 1)
        );
    END COMPONENT;
    

--------------------------------------------------------
--     SIGNALS
--------------------------------------------------------
constant NSDRAM_BYTES: integer := 8;	-- 64-bit wide data bus 
constant NCLKS: integer := 2;				-- 1 CLK
--constant NSDRAM_BYTES: integer := 2;	-- 16-bit wide data bus
--constant NCLKS: integer := 1;				-- 1 CLK
constant CLK_period : time := 7.14 ns;  	-- Clock period  140 MHz  
constant CLK_DDR2_REF_period : time := 8 ns;  -- DDR2 SDRAM Clock frequency  125 MHz  
constant DELAY_SDRAM2FPGA: time := 7.0 ns;	-- FPGA input delay
 
-- clocks / reset
signal CLK : std_logic := '0';
signal CLK_DDR2_REF : std_logic := '0';
signal ASYNC_RESET : std_logic := '0';
signal COUNTER: std_logic_vector(15 downto 0) := (others => '0');

 --Inputs
   signal DATA1_IN : std_logic_vector((16*NSDRAM_BYTES - 1) downto 0) := (others => '0');
   signal DATA1_IN_EN : std_logic_vector(1 downto 0) := (others => '0');
   signal DATA1_IN_SETADDR : std_logic := '0';
   signal DATA1_IN_STARTADDR : std_logic_vector(31 downto 0) := (others => '0');
   signal DATA1_IN_CEILINGADDR : std_logic_vector(31 downto 0) := (others => '0');
   signal DATA1_IN_MODE : std_logic_vector(1 downto 0) := (others => '0');
   signal DATA1_OUT_CTS : std_logic := '0';
   signal DATA1_OUT_SETADDR : std_logic := '0';
   signal DATA1_OUT_STARTADDR : std_logic_vector(31 downto 0) := (others => '0');
   signal DATA1_OUT_CEILINGADDR : std_logic_vector(31 downto 0) := (others => '0');
   signal DATA1_OUT_MODE : std_logic_vector(1 downto 0) := (others => '0');

	--BiDirs
   signal SDRAM_DQ : std_logic_vector(63 downto 0);
	signal SDRAM_DQS_P : std_logic_vector(17 downto 0);	-- IMPORTANT! DO NOT INITIALIZE. MICRON SIMULATION NEEDS A CHANGE AT STARTUP
	signal SDRAM_DQS_N : std_logic_vector(17 downto 0);	-- IMPORTANT! DO NOT INITIALIZE. MICRON SIMULATION NEEDS A CHANGE AT STARTUP
	-- same as above but delayed
   signal SDRAM_DQ_D : std_logic_vector(63 downto 0);
   signal SDRAM_DQS_P_D : std_logic_vector(17 downto 0);
   signal SDRAM_DQS_N_D : std_logic_vector(17 downto 0);

 	--Outputs
   signal DATA1_IN_CTS : std_logic;
   signal DATA1_OUT : std_logic_vector((16*NSDRAM_BYTES - 1) downto 0);
   signal DATA1_OUT_VALID : std_logic;
   signal DATA1_OUT_EMPTY : std_logic;
   signal DATA1_IN_ADDR : std_logic_vector(31 downto 0);
   signal DATA1_OUT_ADDR : std_logic_vector(31 downto 0);
   signal SDRAM_ODT : std_logic_vector(1 downto 0);
   signal SDRAM_CK_P : std_logic_vector(1 downto 0) := (others => '0');
   signal SDRAM_CK_N : std_logic_vector(1 downto 0) := (others => '0');
   signal SDRAM_CKE : std_logic_vector(1 downto 0) := (others => '0');
   signal SDRAM_CASN : std_logic;
   signal SDRAM_RASN : std_logic;
   signal SDRAM_WEN : std_logic;
   signal SDRAM_SN : std_logic_vector(1 downto 0);
   signal SDRAM_DM : std_logic_vector((NSDRAM_BYTES - 1) downto 0) := (others => '0');
   signal SDRAM_BA : std_logic_vector(2 downto 0);
   signal SDRAM_A : std_logic_vector(15 downto 0);
   signal SDRAM_READY : std_logic;
   signal SDRAM_TP : std_logic_vector(10 downto 1);

	-- DDR2 module
	signal SDA: std_logic;
	
	-- delayed IO signals
  signal SDRAM_DQS_P_FPGA : std_logic_vector(17 downto 0) := (others => '0');
  signal SDRAM_DQS_N_FPGA : std_logic_vector(17 downto 0) := (others => '0');
  signal SDRAM_DQ_FPGA : std_logic_vector(63 downto 0) := (others => '0');
  signal SDRAM_DQ_DIR: std_logic := '0';
  signal SDRAM_DQS_DIR: std_logic := '0';

-----------------------------------------------------------------
-- IMPLEMENTATION
-----------------------------------------------------------------
BEGIN

   -- Clock process definitions
CLK_process :process
begin
	CLK <= '0';
	wait for CLK_period/2;
	CLK <= '1';
	wait for CLK_period/2;
end process;

CLK_DDR2_REF_process :process
begin
	CLK_DDR2_REF <= '0';
	wait for CLK_DDR2_REF_period/2;
	CLK_DDR2_REF <= '1';
	wait for CLK_DDR2_REF_period/2;
end process;
 
ASYNC_RESET <= '0';	-- reset not mandatory.

-- stimulus scheduler. Counts to 2^15-1 and stays there
SCHEDULER_001: process(CLK)
begin	
	if rising_edge(CLK) then
		if(COUNTER(15) = '0') and (DATA1_IN_CTS = '1') then
			COUNTER <= COUNTER + 1;
		end if;
	end if;
end process;

WRITE_001: process(CLK, COUNTER)
begin	
	if rising_edge(CLK) then
		if(DATA1_IN_CTS = '0') then
			DATA1_IN_EN <= "00";
			DATA1_IN_SETADDR <= '0';	
		else
			if(COUNTER = 8) then
				DATA1_IN_EN <= "00";
				DATA1_IN_SETADDR <= '1';	-- set address one CLK before data
				DATA1_IN_STARTADDR <= x"00000000";
				DATA1_IN_CEILINGADDR <= x"00000008";
				DATA1_IN_MODE <= "10";
			elsif(COUNTER = 9) then
				DATA1_IN_SETADDR <= '0';	
			elsif(COUNTER = 10) then
				DATA1_IN_EN <= "10";
				DATA1_IN <= x"3000A55A2000A55A1000A55A0000A55A";
			elsif(COUNTER = 11) then
				DATA1_IN_EN <= "10";
				DATA1_IN <= x"32345678223456781234567802345678";
			elsif(COUNTER = 12) then
				DATA1_IN_EN <= "10";
				DATA1_IN <= x"39ABCDEF29ABCDEF19ABCDEF09ABCDEF";
			elsif(COUNTER = 13) then
				DATA1_IN_EN <= "10";
				DATA1_IN <= x"37654321276543211765432107654321";
			else
				DATA1_IN_EN <= "00";
				DATA1_IN_SETADDR <= '0';
			end if;
		end if;
	end if;
end process;

READ_001: process(CLK, COUNTER)
begin	
	if rising_edge(CLK) then
		if(COUNTER = 11) then
			DATA1_OUT_STARTADDR <= x"00000000";
			DATA1_OUT_CEILINGADDR <= x"00000008";
--			DATA1_OUT_MODE <= "01";	-- continuous mode
			DATA1_OUT_MODE <= "10";	-- single-shot mode
			DATA1_OUT_SETADDR <= '1';
		elsif(COUNTER = 12) then
			DATA1_OUT_SETADDR <= '0';
		else
		end if;
	end if;
end process;

DATA1_OUT_CTS <= '1'; -- requests data from output elastic buffer
 
-- Instantiate the Unit Under Test (UUT)
DDR2_WRAPPER_001: DDR2_WRAPPER 
generic map(
-- PLATFORM => 5,	-- Virtex-5
	PLATFORM => 6,	-- Spartan-6
	NSDRAM_BYTES => NSDRAM_BYTES,	-- 8 = sodimm 8-byte wide data bus
	NCLKS => 2,					-- 2 CLKs (SODIMM)
	ADDRESS_MAP => "10010",	-- 4GB SODIMM module
	CLOCK_CYCLES => 3,		-- CL
	DDR2_CLK_FREQ => 2000,	-- 200 MHz
	RTDELAY_COARSE => 2	-- ideal no round-trip delay fpga-sodimm-fpga
--	NSDRAM_BYTES => NSDRAM_BYTES,	-- 2 = sodimm 8-byte wide data bus
--	NCLKS => 1,					-- 1 CLK
--	ADDRESS_MAP => "00001",	-- 1Gb 16-bit wide IC
--	CLOCK_CYCLES => 3,		-- CL
--	DDR2_CLK_FREQ => 1250,	-- 125 MHz
--	RTDELAY_COARSE => 2	-- ideal no round-trip delay fpga-sodimm-fpga
)
PORT MAP (
	 CLK => CLK,
	 CLK_DDR2_REF => CLK_DDR2_REF,
	 ASYNC_RESET => ASYNC_RESET,
	 DATA1_IN => DATA1_IN,
	 DATA1_IN_EN => DATA1_IN_EN,
	 DATA1_IN_CTS => DATA1_IN_CTS,
	 DATA1_IN_SETADDR => DATA1_IN_SETADDR,
	 DATA1_IN_STARTADDR => DATA1_IN_STARTADDR,
	 DATA1_IN_CEILINGADDR => DATA1_IN_CEILINGADDR,
	 DATA1_IN_MODE => DATA1_IN_MODE,
	 DATA1_OUT => DATA1_OUT,
	 DATA1_OUT_VALID => DATA1_OUT_VALID,
	 DATA1_OUT_CTS => DATA1_OUT_CTS,
	 DATA1_OUT_EMPTY => DATA1_OUT_EMPTY,
	 DATA1_OUT_SETADDR => DATA1_OUT_SETADDR,
	 DATA1_OUT_STARTADDR => DATA1_OUT_STARTADDR,
	 DATA1_OUT_CEILINGADDR => DATA1_OUT_CEILINGADDR,
	 DATA1_OUT_MODE => DATA1_OUT_MODE,
	 DATA1_IN_ADDR => DATA1_IN_ADDR,
	 DATA1_OUT_ADDR => DATA1_OUT_ADDR,
	SDRAM_DQ => SDRAM_DQ_FPGA((8*NSDRAM_BYTES - 1) downto 0),
	SDRAM_DQS_P => SDRAM_DQS_P_FPGA((NSDRAM_BYTES - 1) downto 0),
	SDRAM_DQS_N => SDRAM_DQS_N_FPGA((NSDRAM_BYTES - 1) downto 0),
	SDRAM_ODT => SDRAM_ODT,
	SDRAM_CK_P => SDRAM_CK_P((NCLKS -1) downto 0),
	SDRAM_CK_N => SDRAM_CK_N((NCLKS -1) downto 0),
	SDRAM_CKE => SDRAM_CKE((NCLKS -1) downto 0),
	SDRAM_CASN => SDRAM_CASN,
	SDRAM_RASN => SDRAM_RASN ,
	SDRAM_WEN => SDRAM_WEN,
	SDRAM_SN => SDRAM_SN,
	SDRAM_DM => SDRAM_DM((NSDRAM_BYTES - 1) downto 0),
	SDRAM_BA => SDRAM_BA,
	SDRAM_A => SDRAM_A,
	SDRAM_DQ_DIR => SDRAM_DQ_DIR,
	SDRAM_DQS_DIR => SDRAM_DQS_DIR,
	SDRAM_TP => SDRAM_TP 
  );

-- simulate delay at FPGA input
SDRAM_DQS_P_D((NSDRAM_BYTES - 1) downto 0) <= transport SDRAM_DQS_P((NSDRAM_BYTES - 1) downto 0) after DELAY_SDRAM2FPGA;
SDRAM_DQS_N_D((NSDRAM_BYTES - 1) downto 0) <= transport SDRAM_DQS_N((NSDRAM_BYTES - 1) downto 0) after DELAY_SDRAM2FPGA;
SDRAM_DQ_D((8*NSDRAM_BYTES - 1) downto 0) <= transport SDRAM_DQ((8*NSDRAM_BYTES - 1) downto 0) after DELAY_SDRAM2FPGA;

process(SDRAM_DQ_DIR, SDRAM_DQ_FPGA, SDRAM_DQ_D)
begin
	if(SDRAM_DQ_DIR = '0') then
		-- FPGA to SDRAM direction: no delay (all signals and clocks going in the same direction)
		SDRAM_DQ((8*NSDRAM_BYTES - 1) downto 0) <= SDRAM_DQ_FPGA((8*NSDRAM_BYTES - 1) downto 0);
		SDRAM_DQ_FPGA((8*NSDRAM_BYTES - 1) downto 0) <= (others => 'Z');
	else
		-- SDRAM to FPGA direction. delay.
		SDRAM_DQ((8*NSDRAM_BYTES - 1) downto 0) <= (others => 'Z');
		SDRAM_DQ_FPGA((8*NSDRAM_BYTES - 1) downto 0) <= SDRAM_DQ_D((8*NSDRAM_BYTES - 1) downto 0);
	end if;
end process;

process(SDRAM_DQS_DIR, SDRAM_DQS_P_FPGA, SDRAM_DQS_N_FPGA, SDRAM_DQS_P_D, SDRAM_DQS_N_D)
begin
	if(SDRAM_DQS_DIR = '0') then
		-- FPGA to SDRAM direction: no delay (all signals and clocks going in the same direction)
		SDRAM_DQS_P((NSDRAM_BYTES - 1) downto 0) <=  SDRAM_DQS_P_FPGA((NSDRAM_BYTES - 1) downto 0);
		SDRAM_DQS_N((NSDRAM_BYTES - 1) downto 0) <=  SDRAM_DQS_N_FPGA((NSDRAM_BYTES - 1) downto 0);
		SDRAM_DQS_P_FPGA((NSDRAM_BYTES - 1) downto 0) <=  (others => 'Z');
		SDRAM_DQS_N_FPGA((NSDRAM_BYTES - 1) downto 0) <=  (others => 'Z');
	else
		-- SDRAM to FPGA direction. delay.
		SDRAM_DQS_P((NSDRAM_BYTES - 1) downto 0) <=  (others => 'Z');
		SDRAM_DQS_N((NSDRAM_BYTES - 1) downto 0) <=  (others => 'Z');
		SDRAM_DQS_P_FPGA((NSDRAM_BYTES - 1) downto 0) <=  SDRAM_DQS_P_D((NSDRAM_BYTES - 1) downto 0);
		SDRAM_DQS_N_FPGA((NSDRAM_BYTES - 1) downto 0) <=  SDRAM_DQS_N_D((NSDRAM_BYTES - 1) downto 0);
	end if;
end process;


--// DDR2 MODULE SIMULATION (MICRON)
SDRAM_DQS_P(17 downto NSDRAM_BYTES) <= (others => '0');
SDRAM_DQS_N(17 downto NSDRAM_BYTES) <= (others => '0');

Inst_ddr2_module: ddr2_module PORT MAP(
	ck => SDRAM_CK_P,
	ck_n => SDRAM_CK_N,
	cke => SDRAM_CKE,
	s_n => SDRAM_SN,
	ras_n => SDRAM_RASN,
	cas_n => SDRAM_CASN,
	we_n => SDRAM_WEN,
	ba => SDRAM_BA,
	addr => SDRAM_A,
	odt => SDRAM_ODT,
	dqs => SDRAM_DQS_P,
	dqs_n => SDRAM_DQS_N,
	dq => SDRAM_DQ,
	scl => '0',
	sa => "00",
	sda => SDA
);



END;
