-------------------------------------------------------------
--	Filename:  CLKGEN_DCM.VHD
-- Authors: 
--		Alain Zarembowitch / MSS, 
--	Version: 2
--	Date last modified: 2/25/12
-- Inheritance: 	none
--
-- description: generate DDR2 clock phases using a DCM
-- CLK_DDR2G: 140 MHz DDR2 clock 
-- CLK_DDR2_90G: phase-shifted DDR2 clock, (lags)
-- CLK_DDR2_180G: phase-shifted DDR2 clock, (opposite)
-- CLK_DDR2_270G: phase-shifted DDR2 clock, (leads)
--
-- Rev 2 2/25/12 AZ  
-- Added support for V5 platform.
----------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.numeric_std.all;
library unisim;
use unisim.vcomponents.all;

entity CLKGEN_DCM is
Generic (
	PLATFORM: integer := 5
		-- 5 = Xilinx Virtex-5
		-- 6 = Xilinx Spartan-6
		);
port
 (-- Clock in ports (directly from input pin)
  CLK_DDR2_REF           : in     std_logic;
  ASYNC_RESET : in std_logic;
  -- Clock out ports (after BUFGs)
  CLK_DDR2G          : out    std_logic;
  CLK_DDR2_90G          : out    std_logic;
  CLK_DDR2_180G          : out    std_logic;
  CLK_DDR2_270G          : out    std_logic;
  -- Status and control signals
  LOCKED_DCM            : out    std_logic
 );
end CLKGEN_DCM;

architecture Behavioral of CLKGEN_DCM is
--------------------------------------------------------
--     SIGNALS
--------------------------------------------------------
signal CLK_DDR2: std_logic;
signal CLK_DDR2_90: std_logic;
signal CLK_DDR2_180: std_logic;
signal CLK_DDR2_270: std_logic;
signal CLK_DDR2G_LOCAL: std_logic;

--------------------------------------------------------
--      IMPLEMENTATION
--------------------------------------------------------
begin





-- Create 4 clock phases (90 deg apart) for the DDR2 SDRAM clock


DCM_SP_GEN_S6A: if (PLATFORM = 6) generate
   -- DCM_SP: Digital Clock Manager
   --         Spartan-6
   -- Xilinx HDL Language Template, version 11.4

   DCM_SP_inst : DCM_SP
   generic map (
      CLKDV_DIVIDE => 2.0,                   -- CLKDV divide value
                                             -- (1.5,2,2.5,3,3.5,4,4.5,5,5.5,6,6.5,7,7.5,8,9,10,11,12,13,14,15,16).
      CLKFX_DIVIDE => 1,                     -- Divide value - D - (1-32)
      CLKFX_MULTIPLY => 2,                   -- Multiply value - M - (2-32)
      CLKIN_DIVIDE_BY_2 => FALSE,            -- CLKIN divide by two (TRUE/FALSE)
      CLKIN_PERIOD => 5.0,                	-- Input clock period specified in nS
      CLKOUT_PHASE_SHIFT => "NONE",          -- Output phase shift (NONE, FIXED, VARIABLE)
      CLK_FEEDBACK => "1X",                  -- Feedback source (NONE, 1X, 2X)
      DESKEW_ADJUST => "SYSTEM_SYNCHRONOUS", -- SYSTEM_SYNCHRNOUS or SOURCE_SYNCHRONOUS
      DFS_FREQUENCY_MODE => "LOW",           -- Unsupported - Do not change value
      DLL_FREQUENCY_MODE => "LOW",           -- Unsupported - Do not change value
      DSS_MODE => "NONE",                    -- Unsupported - Do not change value
      DUTY_CYCLE_CORRECTION => TRUE,         -- Unsupported - Do not change value
      FACTORY_JF => X"c080",                 -- Unsupported - Do not change value
      PHASE_SHIFT => 0,                      -- Amount of fixed phase shift (-255 to 255)
      STARTUP_WAIT => FALSE                  -- Delay config DONE until DCM LOCKED (TRUE/FALSE)
   )
   port map (
      CLK0 => CLK_DDR2,         	-- 1-bit 0 degree clock output
      CLK90 => CLK_DDR2_90,      -- 1-bit 90 degree clock output
      CLK180 => CLK_DDR2_180,    -- 1-bit 180 degree clock output
      CLK270 => CLK_DDR2_270,    -- 1-bit 270 degree clock output
      CLK2X => open,       		-- 1-bit 2X clock frequency clock output
      CLK2X180 => open, 			-- 1-bit 2X clock frequency, 180 degree clock output
      CLKDV => open,       		-- 1-bit Divided clock output
      CLKFX => open,       		-- 1-bit Digital Frequency Synthesizer output (DFS)
      CLKFX180 => open, 			-- 1-bit 180 degree CLKFX output
      LOCKED => LOCKED_DCM,     	-- 1-bit DCM Lock Output
      PSDONE => open,     			-- 1-bit Phase shift done output
      STATUS => open,     			-- 8-bit DCM status output
      CLKFB => CLK_DDR2G_LOCAL,       	-- 1-bit Clock feedback input
      CLKIN => CLK_DDR2_REF,     -- 1-bit Clock input from PLL above
      DSSEN => '0',       			-- 1-bit Unsupported
      PSCLK => '0',       			-- 1-bit Phase shift clock input
      PSEN => '0',         		-- 1-bit Phase shift enable
      PSINCDEC => '0', 				-- 1-bit Phase shift increment/decrement input
      RST => ASYNC_RESET         -- 1-bit Active high reset input
   );
end generate;

DCM_SP_GEN_V5A: if (PLATFORM = 5) generate
   -- DCM_BASE: Base Digital Clock Manager Circuit
   --           Virtex-5
   -- Xilinx HDL Language Template, version 13.4

   DCM_BASE_inst : DCM_BASE
   generic map (
      CLKDV_DIVIDE => 2.0, -- Divide by: 1.5,2.0,2.5,3.0,3.5,4.0,4.5,5.0,5.5,6.0,6.5
                           --   7.0,7.5,8.0,9.0,10.0,11.0,12.0,13.0,14.0,15.0 or 16.0
      CLKFX_DIVIDE => 1,   -- Can be any integer from 1 to 32
      CLKFX_MULTIPLY => 2, -- Can be any integer from 2 to 32
      CLKIN_DIVIDE_BY_2 => FALSE, -- TRUE/FALSE to enable CLKIN divide by two feature
      CLKIN_PERIOD => 5.0, -- Specify period of input clock in ns from 1.25 to 1000.00
      CLKOUT_PHASE_SHIFT => "NONE", -- Specify phase shift mode of NONE or FIXED
      CLK_FEEDBACK => "1X",         -- Specify clock feedback of NONE or 1X
      DCM_PERFORMANCE_MODE => "MAX_SPEED",   -- Can be MAX_SPEED or MAX_RANGE
      DESKEW_ADJUST => "SYSTEM_SYNCHRONOUS", -- SOURCE_SYNCHRONOUS, SYSTEM_SYNCHRONOUS or
                                             --   an integer from 0 to 15
      DFS_FREQUENCY_MODE => "LOW",   -- LOW or HIGH frequency mode for frequency synthesis
      DLL_FREQUENCY_MODE => "LOW",   -- LOW, HIGH, or HIGH_SER frequency mode for DLL
      DUTY_CYCLE_CORRECTION => TRUE, -- Duty cycle correction, TRUE or FALSE
      FACTORY_JF => X"F0F0",          -- FACTORY JF Values Suggested to be set to X"F0F0" 
      PHASE_SHIFT => 0, -- Amount of fixed phase shift from -255 to 1023
      STARTUP_WAIT => FALSE) -- Delay configuration DONE until DCM LOCK, TRUE/FALSE
   port map (
      CLK0 => CLK_DDR2,         	-- 1-bit 0 degree clock output
      CLK90 => CLK_DDR2_90,      -- 1-bit 90 degree clock output
      CLK180 => CLK_DDR2_180,    -- 1-bit 180 degree clock output
      CLK270 => CLK_DDR2_270,    -- 1-bit 270 degree clock output
      CLK2X => open,       		-- 1-bit 2X clock frequency clock output
      CLK2X180 => open, 			-- 1-bit 2X clock frequency, 180 degree clock output
      CLKDV => open,       		-- 1-bit Divided clock output
      CLKFX => open,       		-- 1-bit Digital Frequency Synthesizer output (DFS)
      CLKFX180 => open, 			-- 1-bit 180 degree CLKFX output
      LOCKED => LOCKED_DCM,     	-- 1-bit DCM Lock Output
      CLKFB => CLK_DDR2G_LOCAL,       	-- 1-bit Clock feedback input
      CLKIN => CLK_DDR2_REF,     -- 1-bit Clock input from PLL above
      RST => ASYNC_RESET         -- 1-bit Active high reset input
   );
end generate;

  -- Output buffering
  -------------------------------------
BUFG_002: BUFG port map(I => CLK_DDR2, O => CLK_DDR2G_LOCAL);
BUFG_003: BUFG port map(I => CLK_DDR2_90, O => CLK_DDR2_90G);
BUFG_004: BUFG port map(I => CLK_DDR2_180, O => CLK_DDR2_180G);
BUFG_005: BUFG port map(I => CLK_DDR2_270, O => CLK_DDR2_270G);

CLK_DDR2G <= CLK_DDR2G_LOCAL;

end Behavioral;
